<%
'*************************************************************
'*  VB Script Vector Class V1.0
'*
'*  Copyright (C) 2003, Neal Clark, Clark Multimedia
'*  http://www.clarkmultimedia.com
'*
'*  Suggestions and comments should be sent to:
'*
'*  info@clarkmultimedia.com
'* 
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License as published by the Free Software Foundation; either
'*  version 2.1 of the License, or (at your option) any later version.
'* 
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*  
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'*
'*  MODIFICATION HISTORY:
'*  1.0    10-Feb-2002 - Neal Clark (info@clarkmultimedia.com) - Initial Version (VB/ASP code)
'*
'*  TODO:
'*  1. Implement Sort() using Quicksort Algorithm
'*	2. Add Search() and Contains() methods
'*	3. Add toArray(), fromArray() method
'*	4. Implement more efficient resize method
'*************************************************************

class vector
' ************ Declare Private Varialbes ************
	private m_size		'int
	private m_items		'int
	private m_vec()		'array of objects 
	
' ************ Initialize Class ************
	private sub Class_Initialize()
		m_size = 10
		m_items = 0
		redim m_vec(m_size)
  	end sub

' ************ Public Properties ************

'*************************************************************
'* POST: the size of the internal array has been returned
'*************************************************************
	public property get size()
		size = m_size
	end property

'*************************************************************
'* POST: the size of the vector has been set to sizeIn
'*************************************************************
	public property let size(byVal sizeIn)
		if m_size <> sizeIn then
			m_size = sizeIn
			reDim preserve m_vec(m_size)
		end if
	end property

'*************************************************************
'* POST: number of elements in the vector has been returned
'*************************************************************
	public property get items()
		items = m_items
	end property
	
' ************ Public Functions ************

'*************************************************************
'* POST: every element has been removed from the vector
'*************************************************************
	public sub clear()
		reDim m_vec(m_size)
		m_items = 0
	end sub

'*************************************************************
'* POST: the specified object has been added to the end of 
'*		 the vector
'*************************************************************
	public sub add(objectIn)
		if m_items >= m_size then
			increaseCapacity()
		end if
		m_vec(m_items) = objectIn
		m_items = m_items + 1
	end sub

'*************************************************************
'* PRE: 0 <= indexIn <= items
'* POST: object has been added to the vector at the specified
'*		 position all elements below have been shifed down.
'*************************************************************
	public sub insert(indexIn, objectIn)
		dim tmp
		dim i
		
		if m_items >= m_size then
			increaseCapacity()
		end if

		tmp = m_vec(indexIn)
		m_vec(indexIn) = objectIn
		
		for i = indexIn to m_items - 2
			tmp = m_vec(i + 1)
			m_vec(i + 1) = m_vec(i)
			m_vec(i) = tmp
		next
		
		m_items = m_items + 1
	end sub

'*************************************************************
'* PRE: 0 <= indexIn < items
'* POST: returns the element at the specified index
'*************************************************************	
	public function elementAt(indexIn)
		elementAt = m_vec(indexIn)
	end function
	
'*************************************************************
'* PRE: 0 <= indexIn < items
'* POST: element at the specified index as been updated
'*************************************************************	
	public sub update(indexIn, objectIn)
		m_vec(indexIn) = objectIn
	end sub

'*************************************************************
'* POST: returns true if this vector is empty
'*************************************************************	
	public function isEmpty()
		isEmpty = m_items <= 0
	end function

'*************************************************************
'* PRE: 0 <= indexIn < items
'* POST: removes the specified item from the vector
'*************************************************************		
	public sub remove(indexIn)
		dim i
		for i = indexIn to m_items - 2
			m_vec(i) = m_vec(i + 1)
		next
		m_items = m_items - 1
	end sub

'*************************************************************
'* POST: returns true if the specified object exists
'*************************************************************	
	public function contains(byVal o)
		contains = false
		dim i
		for i = 0 to items() - 1
			if o = m_vec(i) then
				contains = true
			end if
		next 
	end function

'*************************************************************
'* POST: returns the position of the element in the vector
'*		 returns -1 if the specified element can't be found
'*************************************************************			
	public function indexOf(byVal o)
		indexOf = -1
		dim i
		for i = 0 to items() - 1
			if o = m_vec(i) then
				indexOf = i
			end if
		next 
	end function
' ************ Private Functions ************

'*************************************************************
'* POST: increased the capacity of the vector by 50%
'*************************************************************	
	private sub increaseCapacity()
		m_size = cInt(m_size + (m_size / 2))
		reDim preserve m_vec(m_size)
	end sub
	
' ************ Terminate Class ************	
  	private sub Class_Terminate()
	
  	end sub
	
end class
%>